#include <sys/types.h>  // ftok(), shmat(), shmdt(), select()
#include <sys/ipc.h>    // ftok(), shmget(), shmctl()
#include <sys/shm.h>    // shmget(), shmat(), shmdt(), shmctl()
#include <sys/time.h>   // select()
#include <unistd.h>     // select()
#include <errno.h>      // errno
#include <string.h>     // strerror()
#include <stdio.h>      // printf()
#include <stdlib.h>     // exit()

#include "app.h"

int main()
{
    key_t           our_key;
    int             our_id;
    void           *our_mem;
    mibval         *mibptr;
    unsigned long   count,  gauge;
    fd_set          fds,    rfds;
    struct timeval  tv;
    int             iret;

    printf("This is an application for the SAMPLE-SUBAGENT.\n");

    printf("   Create a SHARED-MEMORY...");

    our_key = ftok(KEYPATH, KEYID);
    if(our_key == -1){
        printf("ftok() error: errno=%d[%s]\n", errno, strerror(errno));
        exit(1);
    }

    our_id = shmget(our_key, sizeof(mibval), IPC_CREAT|IPC_EXCL|0666);
    if((our_id == -1) && (errno == EEXIST)){
        printf("seems already made...");
        our_id = shmget(our_key, sizeof(mibval), 0);
    }
    if(our_id == -1){
        printf("shmget() error: errno=%d[%s]\n", errno, strerror(errno));
        exit(1);
    }

    our_mem = shmat(our_id, NULL, 0);
    if(our_mem == (void *)-1){
        printf("shmat() error: errno=%d[%s]\n", errno, strerror(errno));
        exit(1);
    }

    printf("succeeded.\n");

    mibptr = (mibval *)our_mem;
    count = mibptr->count;
    gauge = mibptr->gauge;
    printf("   initial contents: count[%ld] gauge[%ld]\n", count, gauge);

    printf("   Wait for the contents update...(Push Enter to exit)...\n");
    FD_ZERO(&fds);
    FD_SET (0, &fds);
    while(1){
        rfds = fds;
        tv.tv_sec  = 3;
        tv.tv_usec = 0;
        iret = select((0 + 1), &rfds, NULL, NULL, &tv);
        if(iret < 0){
            if(errno == EINTR) continue;
            printf("   select() error: errno=%d[%s]\n", errno, strerror(errno));
            break;
        }else if(iret > 0){
            if(FD_ISSET(0, &rfds)) break;
            continue;
        }

        if((mibptr->count == count) && (mibptr->gauge == gauge)) continue;

        count = mibptr->count;
        gauge = mibptr->gauge;
        printf("   contents updated: count[%ld] gauge[%ld]\n", count, gauge);
        printf("   Wait for the contents update...(Push Enter to exit)...\n");
    }

    printf("\n   Going to end...");
    if(shmdt(our_mem) != 0){
        printf("shmdt() error: errno=%d[%s]\n", errno, strerror(errno));
        exit(1);
    }
    if(shmctl(our_id, IPC_RMID, NULL) != 0){
        printf("shmctl() error: errno=%d[%s]\n", errno, strerror(errno));
        exit(1);
    }

    printf("normal ended.\n");

    exit(0);
}
