#include <sys/types.h>  // ftok(), shmat(), shmdt()
#include <sys/ipc.h>    // ftok(), shmget()
#include <sys/shm.h>    // shmget(), shmat(), shmdt()
#include <errno.h>      // errno
#include <string.h>     // strerror()
#include <stdio.h>      // printf()
#include <stdlib.h>     // exit()

#include "app.h"

int main(int argc, char *argv[])
{
    key_t           our_key;
    int             our_id;
    void           *our_mem;
    mibval         *mibptr;
    unsigned long   count,  gauge;

    printf("This is a utility for the application.\n");

    count = gauge = 0;
    if(argc >= 2) count = atol(argv[1]);
    if(argc >= 3) gauge = atol(argv[2]);
    printf("   Update the SHARED-MEMORY.\n"
           "      count ---> %ld\n"
           "      gauge ---> %ld\n", count, gauge);

    printf("   Attach the SHARED-MEMORY...");

    our_key = ftok(KEYPATH, KEYID);
    if(our_key == -1){
        printf("ftok() error: errno=%d[%s]\n", errno, strerror(errno));
        exit(1);
    }

    our_id = shmget(our_key, sizeof(mibval), 0);
    if(our_id == -1){
        printf("shmget() error: errno=%d[%s]\n", errno, strerror(errno));
        exit(1);
    }

    our_mem = shmat(our_id, NULL, 0);
    if(our_mem == (void *)-1){
        printf("shmat() error: errno=%d[%s]\n", errno, strerror(errno));
        exit(1);
    }

    printf("succeeded.\n");

    mibptr = (mibval *)our_mem;
    mibptr->count = count;
    mibptr->gauge = gauge;
    printf("   Update the SHARED-MEMORY...count[%ld] gauge[%ld]\n"
                                    , mibptr->count, mibptr->gauge);

    printf("   Detach the SHARED-MEMORY...");
    if(shmdt(our_mem) != 0){
        printf("shmdt() error: errno=%d[%s]\n", errno, strerror(errno));
        exit(1);
    }

    printf("normal ended.\n");

    exit(0);
}
